@@ -1,10 +1,11 @@
 # ruff: noqa: D100, D101, D102, D103, D104, D105, D107
 from __future__ import annotations
 
+import inspect
 import weakref
+from asyncio import iscoroutinefunction
 from inspect import signature
-from types import MethodType
-from typing import TYPE_CHECKING, Any, Callable, Generic, cast
+from typing import TYPE_CHECKING, Any, Callable, Coroutine, Generic, cast
 
 from redux.basic_types import (
     Action,
@@ -17,6 +18,8 @@ from redux.basic_types import (
 )
 
 if TYPE_CHECKING:
+    from types import MethodType
+
     from redux.main import Store
 
 
@@ -43,7 +46,12 @@ class Autorun(
         self._store = store
         self._selector = selector
         self._comparator = comparator
-        self._func = func
+        if options.keep_ref:
+            self._func = func
+        elif inspect.ismethod(func):
+            self._func = weakref.WeakMethod(func)
+        else:
+            self._func = weakref.ref(func)
         self._options = options
 
         self._last_selector_result: SelectorOutput | None = None
@@ -51,22 +59,91 @@ class Autorun(
             ComparatorOutput,
             object(),
         )
-        self._latest_value: AutorunOriginalReturnType | None = options.default_value
+        self._latest_value: AutorunOriginalReturnType = options.default_value
         self._subscriptions: set[
             Callable[[AutorunOriginalReturnType], Any]
             | weakref.ref[Callable[[AutorunOriginalReturnType], Any]]
         ] = set()
+        self._immediate_run = (
+            not iscoroutinefunction(func)
+            if options.subscribers_immediate_run is None
+            else options.subscribers_immediate_run
+        )
 
         if self._options.initial_run and store._state is not None:  # noqa: SLF001
-            self.check_and_call(store._state)  # noqa: SLF001
+            self._check_and_call(store._state)  # noqa: SLF001
 
-        store.subscribe(self.check_and_call)
+        store.subscribe(self._check_and_call)
 
-    def check_and_call(self: Autorun, state: State) -> None:
+    def inform_subscribers(
+        self: Autorun[
+            State,
+            Action,
+            Event,
+            SelectorOutput,
+            ComparatorOutput,
+            AutorunOriginalReturnType,
+        ],
+    ) -> None:
+        for subscriber_ in self._subscriptions.copy():
+            if isinstance(subscriber_, weakref.ref):
+                subscriber = subscriber_()
+                if subscriber is None:
+                    self._subscriptions.discard(subscriber_)
+                    continue
+            else:
+                subscriber = subscriber_
+            subscriber(self._latest_value)
+
+    def call_func(
+        self: Autorun[
+            State,
+            Action,
+            Event,
+            SelectorOutput,
+            ComparatorOutput,
+            AutorunOriginalReturnType,
+        ],
+        selector_result: SelectorOutput,
+        previous_result: SelectorOutput | None,
+        func: Callable[
+            [SelectorOutput, SelectorOutput],
+            AutorunOriginalReturnType,
+        ]
+        | Callable[[SelectorOutput], AutorunOriginalReturnType]
+        | MethodType,
+    ) -> AutorunOriginalReturnType:
+        if len(signature(func).parameters) == 1:
+            return cast(
+                Callable[[SelectorOutput], AutorunOriginalReturnType],
+                func,
+            )(selector_result)
+        return cast(
+            Callable[
+                [SelectorOutput, SelectorOutput | None],
+                AutorunOriginalReturnType,
+            ],
+            func,
+        )(selector_result, previous_result)
+
+    def _check_and_call(
+        self: Autorun[
+            State,
+            Action,
+            Event,
+            SelectorOutput,
+            ComparatorOutput,
+            AutorunOriginalReturnType,
+        ],
+        state: State,
+    ) -> None:
         try:
             selector_result = self._selector(state)
         except AttributeError:
             return
+        func = self._func() if isinstance(self._func, weakref.ref) else self._func
+        if func is None:
+            return
         if self._comparator is None:
             comparator_result = cast(ComparatorOutput, selector_result)
         else:
@@ -75,31 +152,11 @@ class Autorun(
             previous_result = self._last_selector_result
             self._last_selector_result = selector_result
             self._last_comparator_result = comparator_result
-            if len(signature(self._func).parameters) == 1:
-                self._latest_value = cast(
-                    Callable[[SelectorOutput], AutorunOriginalReturnType],
-                    self._func,
-                )(selector_result)
+            self._latest_value = self.call_func(selector_result, previous_result, func)
+            if self._immediate_run:
+                self.inform_subscribers()
             else:
-                self._latest_value = cast(
-                    Callable[
-                        [SelectorOutput, SelectorOutput | None],
-                        AutorunOriginalReturnType,
-                    ],
-                    self._func,
-                )(
-                    selector_result,
-                    previous_result,
-                )
-            for subscriber_ in self._subscriptions.copy():
-                if isinstance(subscriber_, weakref.ref):
-                    subscriber = subscriber_()
-                    if subscriber is None:
-                        self._subscriptions.discard(subscriber_)
-                        continue
-                else:
-                    subscriber = subscriber_
-                subscriber(self._latest_value)
+                self._store._create_task(cast(Coroutine, self._latest_value))  # noqa: SLF001
 
     def __call__(
         self: Autorun[
@@ -112,7 +169,7 @@ class Autorun(
         ],
     ) -> AutorunOriginalReturnType:
         if self._store._state is not None:  # noqa: SLF001
-            self.check_and_call(self._store._state)  # noqa: SLF001
+            self._check_and_call(self._store._state)  # noqa: SLF001
         return cast(AutorunOriginalReturnType, self._latest_value)
 
     def __repr__(
@@ -161,7 +218,7 @@ class Autorun(
             keep_ref = self._options.subscribers_keep_ref
         if keep_ref:
             callback_ref = callback
-        elif isinstance(callback, MethodType):
+        elif inspect.ismethod(callback):
             callback_ref = weakref.WeakMethod(callback)
         else:
             callback_ref = weakref.ref(callback)
